﻿-- Create Ace3 instance
SoDEC = LibStub("AceAddon-3.0"):NewAddon("SoDEndlessChat", "AceConsole-3.0", "AceHook-3.0")

-- UpValue
local _G = _G

-- varables
local defaultMaxLetters = 255							
local defaultMaxBytes = 256

local SoDECBNConvo = {}									-- BN Convo Whispers
local SoDECBNWhisper = {}								-- BN Regular Whispers

local UpdateFrame = CreateFrame("frame", nil, UIParent)	-- OnUpdate Frame
local totalBNWhispT, totalBNConvoT = 0, 0				-- OnUpdate Count

local splitMessageLinks = {}							-- Temp For Links
local words = {}										-- Temp For Words
local msgs = {}											-- Messages

------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------- Initialisation Functions --------------------------------------------------------------------
function SoDEC:OnInitialize()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnEnable()
	self:SecureHook("ChatEdit_UpdateHeader", "ChatEdit_Update")
	self:RawHook("SendChatMessage", true)
	self:RawHook("BNSendWhisper", true)
	self:RawHook("BNSendConversationMessage", true)
	
	local editBox = _G.DEFAULT_CHAT_FRAME.editBox
	defaultMaxLetters = editBox:GetMaxLetters()
	defaultMaxBytes = editBox:GetMaxBytes()	
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnDisable()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnProfileChanged()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------- OnUpdate Function ------------------------------------------------------------------------
local function onUpdate(self, elapsed)
	if SoDECBNWhisper[1] or SoDECBNConvo[1] then
		if SoDECBNWhisper[1] then
			totalBNWhispT = totalBNWhispT + elapsed
			if totalBNWhispT >= .1 then
				totalBNWhispT = 0
				_G.BNSendWhisper(SoDECBNWhisper[1].BNID, SoDECBNWhisper[1].Message)
				tremove(SoDECBNWhisper,1)
			end
		end
		if SoDECBNConvo[1] then
			totalBNConvoT = totalBNConvoT + elapsed
			if totalBNConvoT >= .1 then
				totalBNConvoT = 0
				_G.BNSendConversationMessage(SoDECBNConvo[1].BNID, SoDECBNConvo[1].Message)
				tremove(SoDECBNConvo,1)
			end
		end
	else
		UpdateFrame:SetScript("OnUpdate", nil)
	end
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------- Call Functions -------------------------------------------------------------------------
function SoDEC:ChatEdit_Update(self)
	local activeWindow = _G.ChatEdit_GetActiveWindow()
	if not activeWindow then return end
	
	local chattype = self:GetAttribute("chatType")
	if chattype ~= "YELL" then
		self:SetMaxLetters(0)
		self:SetMaxBytes(0)
	else
		self:SetMaxLetters(defaultMaxLetters)
		self:SetMaxBytes(defaultMaxBytes)
	end
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:BNSendConversationMessage(...)
	local BNETID, text = ...
	if string.len(text) > defaultMaxLetters then
		SoDEC:SplitIntoMsgs(text)
		local n = 1
		for i=2, #msgs do 
			SoDECBNConvo[n] = {BNID = BNETID, Message = msgs[i]}
			n = n + 1
		end
		_G.BNSendConversationMessage(BNETID, msgs[1])
		_G.wipe(msgs)
		UpdateFrame:SetScript("OnUpdate", onUpdate)
		return
	end
	SoDEC.hooks.BNSendConversationMessage(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:BNSendWhisper(...)
	local BNETID, text = ...
	if string.len(text) > defaultMaxLetters then
		SoDEC:SplitIntoMsgs(text)
		local n = 1
		for i=2, #msgs do 
			SoDECBNWhisper[n] = {BNID = BNETID, Message = msgs[i]}
			n = n + 1
		end
		_G.BNSendWhisper(BNETID, msgs[1])
		_G.wipe(msgs)
		UpdateFrame:SetScript("OnUpdate", onUpdate)
		return
	end
	SoDEC.hooks.BNSendWhisper(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:SendChatMessage(...)
	local text, chatType, language, channel = ...
	if string.len(text) > defaultMaxLetters then
		SoDEC:SplitIntoMsgs(text)
		for i=1, #msgs do 
			_G.ChatThrottleLib:SendChatMessage("ALERT", "SoDEC", msgs[i], chatType, language, channel)
		end
		_G.wipe(msgs)
		return
	end
	SoDEC.hooks.SendChatMessage(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------- TOOL Functions -------------------------------------------------------------------------
function SoDEC:SplitIntoMsgs(longText)
	local temp = ""
	-- Parse out links
	longText = string.gsub(longText, "(|H[^|]+|h[^|]+|h)", function(theLink)
				table.insert(splitMessageLinks, theLink)
				return "\001\002"..SoDEC:paddString(#splitMessageLinks, "0", string.len(theLink)-4).."\003\004"
	end)
	
	-- Breakdown Words
	for v in string.gmatch(longText, "[^ ]+") do
		if string.len(v) > defaultMaxLetters then
			local shortPart, remainingPart = nil, v
			local i = 1
			while (remainingPart and string.len(remainingPart) > 0) do
				shortPart, remainingPart = SoDEC:GetShorterString(remainingPart)
				if shortPart and shortPart ~= "" then table.insert(words, shortPart) end
				if i > 10 then break end
				i = i + 1
			end
		else table.insert(words, v) end
	end
	
	-- Recombine
	for i=1, #words + 1 do 
		if words[i] and (string.len(temp) + string.len(words[i])) <= (defaultMaxLetters - 1) then
			if string.len(temp) > 0 then temp = temp.." "..words[i]
			else temp = words[i] end
		else
			temp = string.gsub(temp, "\001\002%d+\003\004", function(link)
						local index = tonumber(string.match(link, "(%d+)"))
						return splitMessageLinks[index] or link
			end)
			table.insert(msgs, temp)
			if words[i] then temp = words[i] end
		end
	end
	_G.wipe(words)
	_G.wipe(splitMessageLinks)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
-- UTF8 for Localization
function SoDEC:GetShorterString(longText)
	local shortText = string.utf8sub(longText, 1, defaultMaxLetters - 1)
	local remainingPart = string.utf8sub(longText, string.len(shortText) + 1, string.len(longText))
	return shortText, remainingPart
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:paddString(str, paddingChar, minLength, paddRight)
    str = tostring(str or "")
    paddingChar = tostring(paddingChar or " ")
    minLength = tonumber(minLength or 0)
    while string.len(str) < minLength do
        if paddRight then str = str..paddingChar
        else str = paddingChar..str end
    end
    return str
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
SLASH_SODENDLESSCHAT1 = "/cmds"
local function handler(text, editbox)
	for v in string.gmatch(text, "[^/]+") do
		editbox:SetText("/"..v)
		ChatEdit_SendText(editbox)
	end
end
SlashCmdList["SODENDLESSCHAT"] = handler